/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package tabs;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.ResourceBundle;


import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import core.Loader;

/**
 * Instances of this class represents a tab that
 * can be used to free draw on the image
 *
 */
public class FreeDrawTab extends TabContainer {

	/**
	 * The opacity that will be used to draw the color
	 */
	private static float ALPHA_COMPOSITE_OPACITY = 0.2f;

	/**
	 * The stroke's default size
	 */
	private float STROKE_SIZE = 10.0f;

	/**
	 * messages that are to be used by the entire GUI
	 */
	private ResourceBundle messages;

	/**
	 * A list with all the colors used by ColorButton
	 * For each color in this list one ColorButton will be created
	 */
	private Color[] colorsList = {
			new Color(Display.getCurrent(), 236, 229, 236),
			new Color(Display.getCurrent(), 206, 233, 240),
			new Color(Display.getCurrent(), 147, 219, 237),
			new Color(Display.getCurrent(), 80, 192, 220),
			new Color(Display.getCurrent(), 53, 154, 179),
			new Color(Display.getCurrent(), 23, 109, 130),
			new Color(Display.getCurrent(), 229, 201, 187),
			new Color(Display.getCurrent(), 203, 169, 152),
			new Color(Display.getCurrent(), 189, 137, 111),
			new Color(Display.getCurrent(), 166, 104, 73),
			new Color(Display.getCurrent(), 140, 79, 49),
			new Color(Display.getCurrent(), 114, 50, 18),
			new Color(Display.getCurrent(), 250, 193, 210),
			new Color(Display.getCurrent(), 231, 169, 184),
			new Color(Display.getCurrent(), 219, 153, 167),
			new Color(Display.getCurrent(), 196, 119, 135),
			new Color(Display.getCurrent(), 191, 71, 96),
			new Color(Display.getCurrent(), 159, 67, 92),
			new Color(Display.getCurrent(), 238, 205, 226),
			new Color(Display.getCurrent(), 217, 182, 214),
			new Color(Display.getCurrent(), 196, 144, 192),
			new Color(Display.getCurrent(), 193, 103, 186),
			new Color(Display.getCurrent(), 169, 70, 161),
			new Color(Display.getCurrent(), 129, 79, 104),
			new Color(Display.getCurrent(), 247, 185, 196),
			new Color(Display.getCurrent(), 239, 152, 152),
			new Color(Display.getCurrent(), 229, 123, 123),
			new Color(Display.getCurrent(), 207, 82, 94),
			new Color(Display.getCurrent(), 196, 55, 68),
			new Color(Display.getCurrent(), 167, 32, 46),
			new Color(Display.getCurrent(), 255, 190, 172),
			new Color(Display.getCurrent(), 239, 153, 130),
			new Color(Display.getCurrent(), 228, 120, 91),
			new Color(Display.getCurrent(), 213, 102, 73),
			new Color(Display.getCurrent(), 201, 96, 68),
			new Color(Display.getCurrent(), 179, 87, 62),
			new Color(Display.getCurrent(), 241, 209, 158),
			new Color(Display.getCurrent(), 231, 196, 141),
			new Color(Display.getCurrent(), 236, 185, 104),
			new Color(Display.getCurrent(), 234, 160, 44),
			new Color(Display.getCurrent(), 222, 145, 24),
			new Color(Display.getCurrent(), 200, 129, 19),
			new Color(Display.getCurrent(), 194, 241, 159),
			new Color(Display.getCurrent(), 160, 224, 115),
			new Color(Display.getCurrent(), 109, 181, 56),
			new Color(Display.getCurrent(), 62, 124, 17),
			new Color(Display.getCurrent(), 41, 90, 5),
			new Color(Display.getCurrent(), 47, 66, 37),
			new Color(Display.getCurrent(), 212, 212, 212),
			new Color(Display.getCurrent(), 184, 184, 184),
			new Color(Display.getCurrent(), 159, 157, 158),
			new Color(Display.getCurrent(), 114, 114, 114),
			new Color(Display.getCurrent(), 77, 77, 77),
			new Color(Display.getCurrent(), 0, 0, 0)
	};

	/**
	 * Use scribble for doing all drawings
	 */
	private ScribbleUtility scribble = null;

	/**
	 * Used to set the upper, color buttons and lower composites
	 */
	private TabGuiUtility tabGui = null;

	//mouse listeners in the drawing area
	private MouseListener mouseListener;
	private MouseMotionListener mouseMotionListener;

	/**
	 * All the colored buttons in use by this tab
	 */
	private ArrayList<ColorButton> coloredButtonList;

	/**
	 * A reference to this instance class
	 */
	private TabContainer thisFreeDrawTab = this;

	/**
	 * Constructs a new instance of this class given its parent,
	 * a style value describing its behavior and appearance and
	 * this instance's position in the {@link TabsContainer} array
	 *
	 * @param parent a {@link TabsContainer} which will be the parent of the new instance
	 * @param style the style of the tab
	 * @param tabLayerPosition the tab's position in the {@link TabsContainer} array
	 *
	 */
	public FreeDrawTab(TabsContainer parent, int style, int tabLayerPosition) {
		super(parent, style, tabLayerPosition);

		messages = Loader.getMessages();

		//the 2 things needed to be done by every implementation of TabComposite
		super.setTabImage(new Image(Display.getCurrent(), System.getProperty("program.pixmapspath") + "tab_icon_freedraw.png"));
		super.setTabToolTipText(messages.getString("FREEDRAW_TAB_TOOLTIP"));

		initialize();

		addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				for (Color c : colorsList) {
					c.dispose();
				}
			}
		});
	}

	/**
	 * create the tab's GUI and initialize it's variables
	 */
	private void initialize() {

		tabGui = new TabGuiUtility(this);
		scribble = new ScribbleUtility(tabsContainer.RESIZED_IMAGE_WIDTH,tabsContainer.RESIZED_IMAGE_HEIGHT);

		tabGui.createUpperComposite(super.getTabToolTipText());
		coloredButtonList = tabGui.createColorButtonsComposite(colorsList);
		tabGui.createBottomComposite();

		scribble.setCurrentColor(tabGui.getCurrentColor());
		scribble.setStroke(new BasicStroke(STROKE_SIZE,1,1));
		scribble.setAlphaOpacity(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, ALPHA_COMPOSITE_OPACITY));

		//for every colorButton set the mouse down event listener
		for (ColorButton button : coloredButtonList) {
			button.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
				public void mouseUp(org.eclipse.swt.events.MouseEvent e) {
					setCurrentColor(tabGui.getCurrentColor());
				}
			});
		}

		//set the stroke scale listener
		tabGui.getStrokeScale().addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				int strokeInterval = 10;
				double adjustInterval = (((float)tabGui.getStrokeScale().getSelection() / tabGui.getStrokeScale().getMaximum()) * strokeInterval) - (strokeInterval / 2.0);
				setStroke(new BasicStroke((float) (STROKE_SIZE + adjustInterval),1,1));
			}
		});

		//set the alpha scale listener
		tabGui.getAlphaScale().addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				double opacityInterval = 0.3;
				double adjustInterval = (((float)tabGui.getAlphaScale().getSelection() / tabGui.getAlphaScale().getMaximum()) * opacityInterval) - (opacityInterval / 2.0);
				setAlphaOpacity(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, (float)(ALPHA_COMPOSITE_OPACITY + adjustInterval)));
			}
		});
	}

	/**
	 * Change the scribble's current color
	 * @param color the new scribble color
	 */
	private void setCurrentColor(java.awt.Color color) {
		scribble.setCurrentColor(color);
	}

	/**
	 * Sets the stroke used for drawing
	 * @param stroke the new stroke
	 */
	private void setStroke(BasicStroke stroke) {
		scribble.setStroke(stroke);
	}

	/**
	 * Change the scribble drawing opacity
	 * @param ac new opacity
	 */
	private void setAlphaOpacity(AlphaComposite ac) {
		scribble.setAlphaOpacity(ac);
	}

	@Override
	public void clear() {
		scribble.clear();
	}

	@Override
	public void drawCurrentLayer(Graphics2D g2d) {
		scribble.drawCurrentLayer(g2d);
	}

	@Override
	public Object getCurrentState() {
		return scribble.getCurrentState();
	}

	@Override
	public BufferedImage getLayerImage() {
		return scribble.getLayerImage();
	}

	@Override
	public void setCurrentState(Object o) {
		scribble.setCurrentState(o);
	}

	public MouseListener mouseListener() {
		if (mouseListener == null) {
			mouseListener = new MouseListener() {

				//unimplemented methods
				public void mouseClicked(MouseEvent e) {}
				public void mouseExited(MouseEvent e) {}

				public void mouseEntered(MouseEvent e) {
					e.getComponent().setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
				}

				public void mousePressed(MouseEvent e) {
					tabsContainer.getDisplay().syncExec(new Runnable() {
						public void run() {
							tabsContainer.createHistoryStep();
						}
					});
					scribble.mousePressed(e);
					tabsContainer.redrawImageArea(thisFreeDrawTab);
				}

				public void mouseReleased(MouseEvent e) {
					scribble.mouseReleased(e);
					tabsContainer.redrawImageArea(thisFreeDrawTab);
				}
			};
		}
		return mouseListener;
	}

	public MouseMotionListener mouseMotionListener() {
		if (mouseMotionListener == null) {
			mouseMotionListener = new MouseMotionListener() {

				//unimplemented method
				public void mouseMoved(MouseEvent e) {}

				public void mouseDragged(MouseEvent e) {
					scribble.mouseDragged(e);
					tabsContainer.redrawImageArea(thisFreeDrawTab);
				}
			};
		}
		return mouseMotionListener;
	}

	public void originalImageChanged() {
		// TODO Auto-generated method stub
	}

	public void disposeData() {
		scribble.dispose();
	}
}