/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package tabs.hair;

import java.io.File;

import org.eclipse.swt.graphics.Image;

/**
 * Instances of this class represent a
 * hair style image read from a XML file
 */
public final class HairStyleData {

	/**
	 * The thumbnail version of the hair
	 */
	private final Image thumbnail;

	/**
	 * The XML file containing all the hair style data
	 */
	private final File hairStyleFile;

	/**
	 * The name of this hair style
	 */
	//TODO this name should be translated, for this version hairstyles have no name
	private final String hairStyleName;

	/**
	 * The name of the owner of this hair
	 */
	private final String ownerName;

	/**
	 * The gender of this hair (if this hair is for male or female)
	 */
	private final String gender;

	/**
	 * The untranslated code for the type of the hair (short/medium/long/etc..)
	 */
	private final String hairType;

	/**
	 * Creates a new instance of this class given all the hairstyle data
	 * @param hairStyleName the hair style's name
	 * @param ownerName the owner's name
	 * @param gender the owner's gender
	 * @param hairType the untranslated code for the type of the hair
	 * @param thumbnail a small thumbnail with the hair style
	 * @param hairStyleFile the file with all the hair style data
	 */
	public HairStyleData(String hairStyleName, String ownerName, String gender, String hairType, Image thumbnail, File hairStyleFile) {
		this.hairStyleName = hairStyleName;
		this.ownerName = ownerName;
		this.gender = gender;
		this.hairType = hairType;
		this.thumbnail = thumbnail;
		this.hairStyleFile = hairStyleFile;
	}

	/**
	 * The file that was used to create this hair style data
	 * @return the file that was used to create this hair style data
	 */
	public File getHairStyleFile() {
		return hairStyleFile;
	}

	/**
	 * A thumbnail containing this hair style which
	 * may differ from the original large hair image
	 * @return thumbnail image of the hair
	 */
	public Image getThumbnail() {
		return thumbnail;
	}

	/**
	 * Whether if this hair is for:<ul>
	 * <li>male</li>
	 * <li>female</li></ul>
	 * @return a string representation of the gender
	 */
	public String getGender() {
		return gender;
	}

	/**
	 * The name of this hair style
	 * @return the name of this hair style
	 */
	public String getHairStyleName() {
		return hairStyleName;
	}

	/**
	 * The untranslated code for the type of the hair
	 * @return the untranslated code for the type of the hair
	 */
	public String getHairType() {
		return hairType;
	}

	/**
	 * The name of the owner of this hair
	 * @return the name of the owner of this hair
	 */
	public String getOwnerName() {
		return ownerName;
	}
}